# Alignment constants
ALIGN_BOTTOM_LEFT = 1
ALIGN_BOTTOM_CENTER = 2
ALIGN_BOTTOM_RIGHT = 3
ALIGN_MIDDLE_LEFT = 4
ALIGN_MIDDLE_CENTER = 5
ALIGN_MIDDLE_RIGHT = 6
ALIGN_TOP_LEFT = 7
ALIGN_TOP_CENTER = 8
ALIGN_TOP_RIGHT = 9

# Legacy alignment mapping
LEGACY_ALIGNMENT_TO_REGULAR = {
    "1": ALIGN_BOTTOM_LEFT,
    "2": ALIGN_BOTTOM_CENTER,
    "3": ALIGN_BOTTOM_RIGHT,
    "5": ALIGN_TOP_LEFT,
    "6": ALIGN_TOP_CENTER,
    "7": ALIGN_TOP_RIGHT,
    "9": ALIGN_MIDDLE_LEFT,
    "10": ALIGN_MIDDLE_CENTER,
    "11": ALIGN_MIDDLE_RIGHT,
}
